#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <math.h>
#include "8x8_horizontal_LSB_1.h"
#include "global.h"
#include "lcd.h"

unsigned char Spalte, Zeile;
const gram pixel = (gram)0x8000;


void lcd_init(void)
{
	MCUCR=1<<SRE;		//Enable XMEM
	

#if (((XTAL_CPU/(FPS*YSIZE))-1)<255)	//line freq	
	TCCR0=(1<<WGM01)|1;
	OCR0=(XTAL_CPU/(FPS*YSIZE))-1;
#elif (((XTAL_CPU/8/(FPS*YSIZE))-1)<255)	
	TCCR0=(1<<WGM01)|2;
	OCR0=(XTAL_CPU/8/(FPS*YSIZE))-1;
#elif (((XTAL_CPU/64/(FPS*YSIZE))-1)<255)	
	TCCR0=(1<<WGM01)|3;
	OCR0=(XTAL_CPU/64/(FPS*YSIZE))-1;
#elif (((XTAL_CPU/256/(FPS*YSIZE))-1)<255)	
	TCCR0=(1<<WGM01)|4;
	OCR0=(XTAL_CPU/256/(FPS*YSIZE))-1;
#elif (((XTAL_CPU/1024/(FPS*YSIZE))-1)<255)	
	TCCR0=(1<<WGM01)|5;
	OCR0=(XTAL_CPU/1024/(FPS*YSIZE))-1;
#endif

	TIMSK=(1<<OCIE0);

#if (SHIFT==2)
	SPCR=(1<<SPE)|(1<<MSTR)|(1<<DORD)|(0<<SPR0)|(POL<<CPOL)|(PHASE<<CPHA);	// fclk/2
	SPSR=(1<<SPI2X);	//10MHz Pixeltakt
#elif (SHIFT==4)
	SPCR=(1<<SPE)|(1<<MSTR)|(1<<DORD)|(0<<SPR0)|(POL<<CPOL)|(PHASE<<CPHA);	// fclk/4
	SPSR=(0<<SPI2X);	//5MHz Pixeltakt
#elif (SHIFT==8)
	SPCR=(1<<SPE)|(1<<MSTR)|(1<<DORD)|(1<<SPR0)|(POL<<CPOL)|(PHASE<<CPHA);	// fclk/8
	SPSR=(1<<SPI2X);	//2,5MHz Pixeltakt
#else
	#error Ungltiger Pixeltakt !
#endif
}

void lcd_setpixel (unsigned char x, unsigned char y)	//Set Pixel
{
	if ((x<XSIZE)&&(y<YSIZE))
	{	pixel[y][x/8]|=1<<(x&7);
	}
}

void lcd_clrpixel (unsigned char x, unsigned char y)	//Clear Pixel
{
	if ((x<XSIZE)&&(y<YSIZE))
	{	pixel[y][x/8]&=~(1<<(x&7));
	}
}

void lcd_writechar (unsigned char Zeichen)				//Display Character    
{
	if ((Spalte<Spalten)&&(Zeile<Zeilen))
	{	unsigned char y;
		for(y=0; y<8; y++)
			pixel[Zeile*8+y][Spalte]=pgm_read_byte(&font[Zeichen][y]);
	}

	Spalte++;
	if (Spalte>=Spalten)
	{	Spalte=0;
		Zeile++;
		if (Zeile>=Zeilen)			
			Zeile=0;
	}
}

void lcd_clear(void)
{	unsigned char x,y;
	for (y=0; y<Zeilen; y++)
		for (x=0; x<XSIZE/8; x++)
			pixel[y][x]=0;
}

SIGNAL(SIG_OUTPUT_COMPARE0)
{	static unsigned char line;
	unsigned char i;

	cbi(LCD_PORT,FLM);
	line++;
	if (line>=YSIZE)			//Start new Frame ?
	{	sbi(LCD_PORT, FLM);		//Set Firstline Marker
		line=0;
	}

	for (i=0; i<XSIZE/8; i++)	//Shiftout Data
	{	SPDR=pixel[line][i];
		while (!(SPSR&(1<<SPIF)));
	}

	sbi (LCD_PORT, LATCH);		//Latch Data
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");
	if (line==0)
		LCD_PORT^=(1<<M);		//Toggle M Signal
	cbi (LCD_PORT, LATCH);	
}

